from __future__ import absolute_import, division, print_function, unicode_literals
from builtins import input

import argparse
import sys

from publisher.git_handling.setup import setup_users_machine
from publisher.utils import get_procedure_code, tspublisher_up_to_date
from publisher.translations.mo import build_mo_files
from publisher.translations.pot import ready_for_translations
from publisher.translations.po import merge_procedure_translation_files
from publisher import procedure_content
from publisher.exceptions import ValidationError
from publisher.processing import procedure
from publisher.settings import CHANNEL_TYPES
from publisher.exceptions import ChannelExistsError, ChannelMissingError


class PublisherCLI():

    def __init__(self, args):
        # Check version is up to date
        if not tspublisher_up_to_date():
            print('######################################################')
            print('Your version of the touch surgery publishing tools \n'
                  'is out of date. Please update using the following \n'
                  'command: \n \n'
                  'pip install -U tspublisher \n')
            print('######################################################')
            sys.exit()

        parser = argparse.ArgumentParser(
            description='Touch Surgery Publisher',
            usage='tspub <command> [<args>]'
        )
        subparsers = parser.add_subparsers(
            title='Available commands',
            description=(
                'These commands allow you to create and '
                'publish TS simulations'
            ),
            help='subparsers help'
        )
        cli_commands = [c for c in dir(self) if not c.startswith('_')]

        for command in cli_commands:
            sub_parser = subparsers.add_parser(
                command,
                help=getattr(self, command).__doc__
            )
            sub_parser.set_defaults(func=getattr(self, command))

        namespace = parser.parse_args(args[0:1])

        try:
            namespace.func(args[1:])
        except ValidationError as e:
            print('The following issues were encountered.')
            print(e.message)

    def setup(self, args):
        """ Setup your machine, ready to create and edit simulations
        """
        setup_users_machine()

    def procedures(self, args):
        """ List all procedures in the git repository, each being a separate branch
        """
        procedure_content.display_procedures()

    def phases(self, args):
        """ List all phases of the procedure and their commit histories.
        """
        phase_code = procedure_content.get_user_selected_phase()
        procedure_content.display_phase_history(phase_code)

    def reset(self, args):
        """ Reset all local changes to match the most recent commit
        """
        procedure_content.cl_reset_repo()

    def revert_phase(self, args):
        """ Revert to a previous commit state for a chosen commit and phase.
        """
        phase_code = procedure_content.get_user_selected_phase()
        procedure_content.display_phase_history_and_revert(phase_code)

    def workon(self, args):
        """ Move to the branch containing the specified procedure so that it can be worked on
        """
        parser = argparse.ArgumentParser(
            description='Move to the branch containing the specified procedure',
            usage='''tspub workon <procedure_code>'''
        )
        parser.add_argument(
            'procedure',
            help='The procedure code'
        )

        self._clean_working_directory_state()

        args = parser.parse_args(args)
        procedure_content.change_procedure(args.procedure)

    def save(self, args):
        """ Commit and push current changes to the repo
        """
        parser = argparse.ArgumentParser(
            description='Commit and push current changes to the repo',
            usage='''tspub save <commit_message>'''
        )
        parser.add_argument(
            'commit_message',
            help='Describe what changes have been made'
        )
        args = parser.parse_args(args)
        procedure_content.save_working_changes(args.commit_message)

    def create(self, args):
        """Create a new branch with the name of the procedure_code param
        """
        parser = argparse.ArgumentParser(
            description='Create new procedure with the specified procedure_code param',
            usage='''tspub create <procedure_code>'''
        )
        parser.add_argument('procedure_code', help='code of the procedure to be created')
        args = parser.parse_args(args)

        if args.procedure_code not in procedure_content.build_procedure_list():
            self._clean_working_directory_state()
            procedure_content.create_procedure_branch(args.procedure_code)
        else:
            raise Exception("The procedure {0} already exists and thus cannot be created".format(args.procedure_code))

        procedure.create_procedure(args.procedure_code)

        procedure_content.save_working_changes("Initial commit", initial=True, procedure_code=args.procedure_code)

    def update_procedure(self, args):
        """Update current working procedure. Update phases with build_phases param.
        """
        parser = argparse.ArgumentParser(
            description='Update current procedure.',
            usage='''tspub update_procedure'''
        )
        parser.add_argument('--build_phases', action='store_true', help='Build all phases as well as procedure')
        parser.add_argument('--graphics', action='store_true', help='Copy latest images (normal)')
        parser.add_argument('--pip_graphics', action='store_true', help='Copy latest pip images (normal)')

        args = self._get_build_args(parser, args)

        procedure_code = get_procedure_code()

        graphics = args.graphics
        if args.graphics:
            graphics = "latest"

        pip_graphics = args.pip_graphics
        if args.pip_graphics:
            pip_graphics = {}

        procedure.build_procedure(procedure_code, build_phases=args.build_phases, graphics=graphics,
                                  pip_graphics=pip_graphics, widget_graphics=args.widget_graphics,
                                  thumbnails=args.thumbnails, step_numbers=args.step_numbers,
                                  info_step=args.info_step, country_restriction=args.country_restrict)

    def update_phase(self, args):
        """Update the specified phase with the phase_code param.
        """
        parser = argparse.ArgumentParser(
            description='Update specified phase with the phase_code param.',
            usage='''tspub update_phase <phase_code>'''
        )
        parser.add_argument('phase_code', help='code of the procedure to be created')
        parser.add_argument('--graphics', help='Copy latest images, can be supplied with version number',
                            const="latest", nargs="?")
        parser.add_argument('--pip_graphics', help='Copy latest pip images, can be supplied with version number e.g.: '
                                                   '--pip_graphics="pipName 2,otherPipName 1"', const={},
                            nargs="?")
        parser.add_argument('--csv_version', help='Define csv version to build with', nargs="+")

        args = self._get_build_args(parser, args)

        pip_graphics = self.format_pip_graphics(args.pip_graphics)

        if args.csv_version:
            csv_version = args.csv_version[0]
        else:
            csv_version = None

        procedure.build_single_phase(args.phase_code, graphics=args.graphics, pip_graphics=pip_graphics,
                                     widget_graphics=args.widget_graphics, thumbnails=args.thumbnails,
                                     step_numbers=args.step_numbers, info_step=args.info_step,
                                     country_restriction=args.country_restrict, csv_version=csv_version)

    def publish(self, args):
        """Publish procedure with the specified distribution group. Defaults to TS-Testing.
        """
        parser = argparse.ArgumentParser(
            description='Publish procedure with the specified distribution group. Defaults to TS-Testing.',
            usage='''tspub publish'''
        )
        parser.add_argument(
            '--distgroup',
            help='Distribution group to be published to e.g. --distgroup=distGroup1,distGroup2',
            required=False
        )
        parser.add_argument(
            '--number',
            help='The number of previous commits to be shown',
            required=False
        )
        args = parser.parse_args(args)
        procedure_content.publish_with_display(args.distgroup, args.number)

    def translate(self, args):
        """Available for testing only, builds a messages.pot in the working directory for the active procedure."""
        if ready_for_translations():
            merge_procedure_translation_files()
            build_mo_files()

    def create_channel(self, args):
        parser = argparse.ArgumentParser()
        parser.add_argument('code')
        parser.add_argument('name')
        parser.add_argument('type', choices=CHANNEL_TYPES)
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument('--eula_required', dest='eula', action='store_true')
        group.add_argument('--eula_not_required', dest='eula', action='store_false')

        args = parser.parse_args(args)
        if args.code in procedure_content.build_channel_list():
            raise ChannelExistsError("A channel with code {} already exists.")
        procedure.build_channel(vars(args))

    def update_channel(self, args):
        parser = argparse.ArgumentParser()
        parser.add_argument('code')
        parser.add_argument('--name')
        parser.add_argument('--type', choices=CHANNEL_TYPES)
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument('--eula_required', dest='eula', action='store_true')
        group.add_argument('--eula_not_required', dest='eula', action='store_false')
        args = parser.parse_args(args)
        if args.code not in procedure_content.build_channel_list():
            raise ChannelMissingError("A channel with code {} does not exist.")

        updates = {}
        if args.name:
            updates['name'] = args.name
        if args.type:
            updates['type'] = args.type
        updates['eula'] = args.eula
        procedure.update_channel(args.code, updates)

    def save_channels(self, args):
        """ Commit and push current channel changes to the repo
        """
        parser = argparse.ArgumentParser(
            description='Commit and push current channel changes to the repo',
            usage='''tspub save_channels <commit_message>'''
        )
        parser.add_argument(
            'commit_message',
            help='Describe what changes have been made'
        )
        args = parser.parse_args(args)
        procedure_content.save_channel_changes(args.commit_message)

    @staticmethod
    def _get_build_args(parser, args):
        parser.add_argument('--widget_graphics', action='store_true', help='Copy latest widget images (normal)')
        parser.add_argument('--thumbnails', action='store_true', help="Pull latest thumbnails (both)")
        parser.add_argument('--step_numbers', action='store_true', help='Prepend step number in step content')
        parser.add_argument('--info_step', action='store_true', help='Add build information')
        parser.add_argument('--country_restrict', default="",
                            help="Comma separated country restrictions (ISO Alpha-2) e.g. US, FR, ES")

        args = parser.parse_args(args)
        return args

    @staticmethod
    def _clean_working_directory_state():
        if procedure_content.has_unstaged_changes():
            user_choice = PublisherCLI._get_user_decision()

            if user_choice == 'y':
                commit_message = PublisherCLI._get_commit_message()
                procedure_content.save_working_changes(commit_message)
            else:
                procedure_content.delete_unstaged_changes()

    @staticmethod
    def _get_commit_message():
        """ Return the commit message from the user
        """
        return input("Please enter a commit message")

    @staticmethod
    def _get_user_decision():
        """ Return the users y or n input
        """
        user_input = ''
        while user_input not in ['y', 'n']:
            print('You have unsaved changes. Would you save them now? If you do not save them they will be deleted')
            print('> y or n:')
            sys.stdout.flush()
            user_input = input('')
        return user_input.lower()

    @staticmethod
    def format_pip_graphics(pip_versions):
        if pip_versions:
            pip_graphics = {}
            pips = pip_versions.split(",")
            for pip in pips:
                version_list = pip.split(" ")
                pip_graphics[version_list[0]] = version_list[1]
            return pip_graphics

        return pip_versions


def main():
    PublisherCLI(sys.argv[1:])



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ts_title.ui'
#
# Created: Tue Aug 14 11:24:16 2018
#
# WARNING! All changes made in this file will be lost!

from Qt import QtWidgets, QtCore, QtGui


class Ui_ts_title(object):
    def setupUi(self, ts_title):
        ts_title.setObjectName("ts_title")
        ts_title.resize(543, 51)
        self.horizontalLayout = QtWidgets.QHBoxLayout(ts_title)
        self.horizontalLayout.setSpacing(12)
        self.horizontalLayout.setContentsMargins(5, -1, -1, -1)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.icon = QtWidgets.QLabel(ts_title)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.icon.sizePolicy().hasHeightForWidth())
        self.icon.setSizePolicy(sizePolicy)
        self.icon.setMinimumSize(QtCore.QSize(32, 32))
        self.icon.setText("")
        self.icon.setPixmap(QtGui.QPixmap(":/icons/icons/TSIcon.png"))
        self.icon.setObjectName("icon")
        self.horizontalLayout.addWidget(self.icon)
        self.label = QtWidgets.QLabel(ts_title)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)

        self.retranslateUi(ts_title)
        QtCore.QMetaObject.connectSlotsByName(ts_title)

    def retranslateUi(self, ts_title):
        ts_title.setWindowTitle(QtWidgets.QApplication.translate("ts_title", "Form", None))
        self.label.setText(QtWidgets.QApplication.translate("ts_title", "Title", None))

from . import icons_rc



import re
from collections import defaultdict

import polib
import six
from ruamel import yaml

from publisher.utils import get_platform, find_command, call_command_and_print_exception
from publisher.processing.utils.file import get_all_procedure_yaml_files


def ready_for_translations():
    if not (find_command('msgmerge') and find_command('msgfmt')):
        if get_platform().startswith('dar'):
            call_command_and_print_exception(['brew', 'install', 'gettext'], 'Could not brew install.')
            call_command_and_print_exception(['brew', 'link', '--force', 'gettext'], 'Could not brew link.')
        else:
            print('You must install gettext to continue with translations. \n'
                  'Please go to https://mlocati.github.io/articles/gettext-iconv-windows.html, \n'
                  'and follow the download instructions.')
            raise EnvironmentError
    return True


def build_translation_template(output_file):
    translatable_strings = get_translatable_strings()
    build_pot_file(translatable_strings, output_file)


def build_pot_file(strings, output_file):
    pot = polib.POFile()
    pot.metadata['Content-Type'] = 'text/plain; charset=UTF-8'
    for string, locations in strings.items():
        entry = polib.POEntry(msgid=string, occurrences=locations)
        pot.append(entry)

    pot.save(fpath=output_file)


def get_translatable_strings():

    results = defaultdict(list)
    content_map = get_content_map()

    for yaml_file in get_all_procedure_yaml_files():
        with open(yaml_file, 'rb') as f:
            yaml_obj = yaml.load(f, Loader=yaml.RoundTripLoader)

        strings = extract_content(yaml_obj, content_map)
        for i in range(len(strings)):
            results[strings[i]].append((yaml_file, i))

    return results


def get_content_map():

    return {
        'name': six.string_types,
        'cta_text': six.string_types,
        'title': six.string_types,
        'subtitle': six.string_types,
        'sub_title': six.string_types,
        'text': six.string_types,
        'content': six.string_types,
        'answer': six.string_types,
        'desc': six.string_types,
        'numbered_list': list,
        'list': list,
        'choices': list,
    }


def extract_content(yaml_obj, content_map):
    results = []
    if isinstance(yaml_obj, list):
        for v in yaml_obj:
            results += extract_content(v, content_map)
    elif isinstance(yaml_obj, dict):
        for k, v in yaml_obj.items():
            mapped_type = content_map.get(k.lower())
            value_matches_type = mapped_type is not None and isinstance(v, mapped_type)
            if value_matches_type:
                string_list = v if mapped_type == list else [v]
                results += [s for s in string_list if is_translatable(s)]
            else:
                results += extract_content(v, content_map)

    return results


def is_translatable(s):

    if s is None or not isinstance(s, six.string_types) or s.strip() == '':
        return False

    try:
        s.encode('ascii')
        is_unicode = False
    except (UnicodeEncodeError, UnicodeDecodeError):
        is_unicode = True

    return is_unicode or (' ' in s or not re.search(r'[\d_]', s))




from Qt import QtWidgets, QtCore

from publisher.gui.delegates import CheckBoxDelegate, ComboBoxDelegate
from publisher.procedure_content import build_channel_list, save_channel_changes
from publisher.processing.data_sources.organisation import get_data_from_channel
from publisher.processing.procedure import update_channel, build_channel
from publisher.settings import CHANNEL_TYPES

from .models import ChannelTableModel
from .ui.channel_editor import Ui_channel_editor
from .widgets import SpinnerOverlay


class ChannelEditor(QtWidgets.QWidget, Ui_channel_editor):

    def __init__(self, parent=None):
        super(ChannelEditor, self).__init__(parent)
        self.channel_model = ChannelTableModel(self)
        self.channel_model.validation_error.connect(self._display_error)
        self.setupUi(self)
        self.channel_tbl.addAction(self.add_row)
        self.channel_tbl.addAction(self.delete_row)
        self.channel_tbl.setModel(self.channel_model)
        self.channel_tbl.setItemDelegateForColumn(self.channel_model.header.index('eula_required'),
                                                  CheckBoxDelegate(self))
        self.channel_tbl.setItemDelegateForColumn(self.channel_model.header.index('type'),
                                                  ComboBoxDelegate(self, CHANNEL_TYPES))
        self.loading_overlay = SpinnerOverlay(self.channel_tbl)
        # Pyside vs PySide2 incompatability not managed correctly by Qt Shim
        if 'setSectionResizeMode' in dir(self.channel_tbl.horizontalHeader()):
            self.channel_tbl.horizontalHeader().setSectionResizeMode(1, QtWidgets.QHeaderView.Stretch)
        else:
            self.channel_tbl.horizontalHeader().setResizeMode(1, QtWidgets.QHeaderView.Stretch)
        self.load_channel_data()

    @QtCore.Slot()
    def on_submit_btn_clicked(self):
        data = self.channel_model.edited_channels
        errors = self._validate_submit(data)
        if errors:
            self._display_error(errors)
            return
        message = self.msg.toPlainText()
        self.data_saver = SaveChannelDataWorker(data, message)
        self.data_saver.done.connect(self._on_write_completed)
        self.data_saver.error.connect(self._display_error)
        self.loading_overlay.show()
        self.data_saver.start()

    @QtCore.Slot()
    def on_add_row_triggered(self):
        data = {'code': '', 'name': '', 'type': CHANNEL_TYPES[0], 'eula_required': False, 'new': True}
        row = self.channel_tbl.selectedIndexes()[0].row()
        self.channel_model.add_channel(data, row)

    @QtCore.Slot()
    def on_delete_row_triggered(self):
        row = self.channel_tbl.selectedIndexes()[0].row()
        self.channel_model.delete_row(row)

    def load_channel_data(self):
        self._disable_input()
        self.loading_overlay.show()
        self.data_loader = LoadChannelDataWorker()
        self.data_loader.channel_data.connect(self.channel_model.add_channel)
        self.data_loader.done.connect(self._on_data_loaded)
        self.data_loader.start()

    def _on_data_loaded(self):
        self.loading_overlay.hide()
        self._enable_input()

    def _on_write_completed(self):
        self.loading_overlay.hide()
        self._enable_input()

    def _display_error(self, message):
        QtWidgets.QMessageBox.critical(self, "Error", "Error: {}".format(message))
        self.loading_overlay.hide()
        self._enable_input()

    def _enable_input(self):
        self.msg.setEnabled(True)
        self.channel_tbl.setEnabled(True)
        self.submit_btn.setEnabled(True)

    def _disable_input(self):
        self.msg.setEnabled(False)
        self.channel_tbl.setEnabled(False)
        self.submit_btn.setEnabled(False)

    def _validate_submit(self, data):
        if self.msg.toPlainText() == '':
            return "Submit comment must not be blank"
        for item in data:
            if not item.get('name'):
                return "Name is blank for channel with code {}".format(item.get('code'))
            if not item.get('code'):
                return "Code is blank for channel with name {}".format(item.get('name'))


class LoadChannelDataWorker(QtCore.QThread):

    channel_data = QtCore.Signal(dict)
    done = QtCore.Signal()

    def run(self):
        for channel in build_channel_list():
            data = get_data_from_channel(channel)
            self.channel_data.emit(data)
        self.done.emit()


class SaveChannelDataWorker(QtCore.QThread):

    done = QtCore.Signal()
    error = QtCore.Signal(str)

    def __init__(self, data, msg):
        try:
            self.data = data
            self.message = msg
            super(SaveChannelDataWorker, self).__init__()
        except Exception as e:
            message = str(e)
            self.error.emit(message)

    def run(self):
        try:
            for channel in self.data:
                if channel.get('new', False):
                    channel_data = {'code': channel['code'], 'name': channel['name'], 'type': channel['type'],
                                    'eula': channel['eula_required']}
                    build_channel(channel_data)
                else:
                    updates = {'name': channel['name'], 'type': channel['type'], 'eula': channel['eula_required']}
                    update_channel(channel['code'], updates)
            save_channel_changes(self.message)
            self.done.emit()
        except Exception as e:
            message = str(e)
            self.error.emit(message)



import os
import subprocess

from ruamel import yaml

import publisher.settings as settings
from publisher.exceptions import LocalesNotSpecifiedError
from publisher.processing.utils.file import get_procedure_yaml_file
from publisher.translations.pot import build_translation_template
from publisher.translations.utils import get_and_check_locale_directory
from publisher.utils import get_procedure_code


def get_procedure_locales():
    procedure_file = get_procedure_yaml_file()

    with open(procedure_file, 'rt') as y:
        yaml_obj = yaml.safe_load(y)

    return yaml_obj.get('locales', [])


def build_procedure_translation_template(procedure_code):
    pot_file = os.path.join(settings.TRANSLATIONS_CHECKOUT_DIRECTORY, '{0}.pot'.format(procedure_code))
    build_translation_template(pot_file)
    return pot_file


def merge_procedure_translation_files():
    procedure_code = get_procedure_code()
    locales = get_procedure_locales()

    template_file = build_procedure_translation_template(procedure_code)

    try:
        if len(locales) == 0:
            raise LocalesNotSpecifiedError()

        for locale in locales:
            output_directory = get_and_check_locale_directory(locale)
            output_file = os.path.join(output_directory, '{0}.po'.format(procedure_code))

            if os.path.exists(output_file):
                subprocess.check_output(['msgmerge', '-U', '-q', '--previous', '--backup=off',
                                         output_file, template_file])
                print('Updated {0}'.format(output_file))
            else:
                subprocess.check_output(['msginit', '--l', locale, '--no-translator',
                                         '-i', template_file, '-o', output_file])
    finally:
        os.remove(template_file)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'channel_editor.ui'
#
# Created: Wed Aug 22 16:50:02 2018
#
# WARNING! All changes made in this file will be lost!

from Qt import QtWidgets, QtCore, QtGui


class Ui_channel_editor(object):
    def setupUi(self, channel_editor):
        channel_editor.setObjectName("channel_editor")
        channel_editor.resize(536, 424)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(channel_editor)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.header = QtWidgets.QFrame(channel_editor)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.header.sizePolicy().hasHeightForWidth())
        self.header.setSizePolicy(sizePolicy)
        self.header.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.header.setFrameShadow(QtWidgets.QFrame.Raised)
        self.header.setObjectName("header")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.header)
        self.verticalLayout.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = TSTitle(self.header)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.line = QtWidgets.QFrame(self.header)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.verticalLayout_2.addWidget(self.header)
        self.channel_tbl = QtWidgets.QTableView(channel_editor)
        self.channel_tbl.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
        self.channel_tbl.setAlternatingRowColors(True)
        self.channel_tbl.setSortingEnabled(True)
        self.channel_tbl.setObjectName("channel_tbl")
        self.channel_tbl.horizontalHeader().setStretchLastSection(False)
        self.channel_tbl.verticalHeader().setVisible(False)
        self.verticalLayout_2.addWidget(self.channel_tbl)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.label_2 = QtWidgets.QLabel(channel_editor)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.msg = QtWidgets.QPlainTextEdit(channel_editor)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.msg.sizePolicy().hasHeightForWidth())
        self.msg.setSizePolicy(sizePolicy)
        self.msg.setMaximumSize(QtCore.QSize(16777215, 50))
        self.msg.setObjectName("msg")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.msg)
        self.verticalLayout_2.addLayout(self.formLayout)
        self.submit_btn = QtWidgets.QPushButton(channel_editor)
        self.submit_btn.setObjectName("submit_btn")
        self.verticalLayout_2.addWidget(self.submit_btn)
        self.add_row = QtWidgets.QAction(channel_editor)
        self.add_row.setObjectName("add_row")
        self.delete_row = QtWidgets.QAction(channel_editor)
        self.delete_row.setObjectName("delete_row")

        self.retranslateUi(channel_editor)
        QtCore.QMetaObject.connectSlotsByName(channel_editor)
        channel_editor.setTabOrder(self.channel_tbl, self.msg)
        channel_editor.setTabOrder(self.msg, self.submit_btn)

    def retranslateUi(self, channel_editor):
        channel_editor.setWindowTitle(QtWidgets.QApplication.translate("channel_editor", "Channel Editor", None))
        self.label.setText(QtWidgets.QApplication.translate("channel_editor", "Channel Editor", None))
        self.label_2.setText(QtWidgets.QApplication.translate("channel_editor", "Comment", None))
        self.submit_btn.setText(QtWidgets.QApplication.translate("channel_editor", "Save Changes", None))
        self.add_row.setText(QtWidgets.QApplication.translate("channel_editor", "Add Row", None))
        self.delete_row.setText(QtWidgets.QApplication.translate("channel_editor", "Delete Row", None))

from publisher.gui.widgets import TSTitle




from Qt import QtWidgets, QtCore, QtGui

from publisher.gui.ui.ts_title import Ui_ts_title


class SpinnerOverlay(QtWidgets.QWidget):

    def __init__(self, parent=None):
        super(SpinnerOverlay, self).__init__(parent)
        self._timer = QtCore.QTimer(self)
        self.box_size = 100
        self.thickness = 6
        self.arc_length = 300
        self._start_angle = 0
        self._timer.timeout.connect(self._on_timer_timeout)

        filter = ResizeEventFilter(parent)
        filter.resized.connect(self._on_parent_resized)
        parent.installEventFilter(filter)

        self.hide()

    def paintEvent(self, event):
        super(SpinnerOverlay, self).paintEvent(event)

        painter = QtGui.QPainter()
        painter.begin(self)
        try:
            painter.setRenderHint(QtGui.QPainter.Antialiasing)

            fg_col = self.palette().color(self.foregroundRole())
            pen = QtGui.QPen(fg_col)
            pen.setWidth(self.thickness)
            painter.setPen(pen)

            width_border = (self.width()-self.box_size)/2
            height_border = (self.height()-self.box_size)/2
            r = QtCore.QRect(0, 0, self.width(), self.height())
            r.adjust(width_border, height_border, -width_border, -height_border)

            # draw the arc:
            painter.drawArc(r, -self._start_angle * 16, self.arc_length * 16)
            r = None

        finally:
            painter.end()
            painter = None

    def showEvent(self, event):
        if not self._timer.isActive():
            self._timer.start(100)
        super(SpinnerOverlay, self).showEvent(event)

    def hideEvent(self, event):
        self._timer.stop()
        super(SpinnerOverlay, self).hideEvent(event)

    def closeEvent(self, event):
        self._timer.stop()
        super(SpinnerOverlay, self).closeEvent(event)

    def _on_timer_timeout(self):
        self._start_angle = (self._start_angle + 10)
        if self._start_angle > 360:
            self._start_angle -= 360
        self.update()

    def _on_parent_resized(self):
        self.resize(self.parentWidget().size())


class TSTitle(QtWidgets.QWidget, Ui_ts_title):

    def __init__(self, parent=None):
        super(TSTitle, self).__init__(parent)
        self.setupUi(self)

    def setText(self, text):
        self.label.setText(text)


class ResizeEventFilter(QtCore.QObject):
    """
    Event filter which emits a resized signal whenever
    the monitored widget resizes. This is so that the overlay wrapper
    class can be informed whenever the Widget gets a resize event.
    """
    resized = QtCore.Signal()

    def eventFilter(self, obj, event):
        if event.type() == QtCore.QEvent.Resize:
            self.resized.emit()
        return False



import os

import publisher.settings as settings
from publisher.exceptions import InvalidLocaleError


def get_and_check_locale_directory(locale):
    locale_directory = get_locale_directory(locale)

    if not os.path.exists(locale_directory):
        os.makedirs(locale_directory)

    return locale_directory


def get_locale_directory(locale):

    if locale not in settings.LOCALE_DIRECTORIES:
        raise InvalidLocaleError(locale)

    return os.path.join(settings.TRANSLATIONS_CHECKOUT_DIRECTORY,
                        settings.LOCALE_DIRECTORIES[locale], 'LC_MESSAGES')



from __future__ import absolute_import, division, print_function, unicode_literals

import sqlite3

import publisher.settings as settings


def get_csv_steps_file(phase_code, csv_version):
    c = sqlite3.connect(settings.CONTENT_DB_DIR)
    try:
        c.text_factory = str
        cursor = c.execute("""
            SELECT f.file_path FROM assetdb_asset a
            INNER JOIN assetdb_assetversion v ON a.id = v.asset_id
            INNER JOIN assetdb_assetfile f ON v.id = f.asset_version_id
            WHERE
                a.category = 'Admin' AND
                a."name" = 'Main' AND
                a."level_of_detail" = 'default' AND
                a."variant" = 'default' AND
                a."module" = ? AND
                a."asset_type" = 'csvSteps' AND
                a."department" = 'default' AND
                a."operation" IS NULL AND
                a."institution" = 'TouchSurgeryContent' AND
                a."stage" = 'default' AND
                NOT a.deprecated AND
                NOT v."pending" AND
                NOT v."deprecated" AND
                NOT f."pending" AND
                NOT f."deprecated" AND
                f."name" = 'default'
            ORDER BY v."version";""", (phase_code,))

        rows = cursor.fetchall()
        converted_rows = [row[0] for row in rows]

        if converted_rows:
            csv_asset = str(_get_csv_version_asset(converted_rows, csv_version))
        else:
            csv_asset = None
    finally:
        c.close()

    return _get_platform_specific_path(csv_asset.strip()) if csv_asset else None


def _get_platform_specific_path(file_path):
    if settings.ORIGINAL_CONTENT_ROOT in file_path:
        file_path = file_path.replace(settings.ORIGINAL_CONTENT_ROOT, settings.REPLACEMENT_CONTENT_ROOT, 1)
    else:
        file_path = file_path.replace(settings.ORIGINAL_CONTENT_ROOT_OLD, settings.REPLACEMENT_CONTENT_ROOT_OLD, 1)

    return file_path.replace("\\", "/")


def _get_csv_version_asset(rows, csv_version):
    if csv_version:
        for row in rows:
            csv_details = _get_platform_specific_path(str(row)).split("/")
            version_number = csv_details[-2]
            if "v%s" % str(csv_version) == version_number:
                return row
        return None
    else:
        return rows[-1]



from __future__ import absolute_import, division, print_function, unicode_literals

import os
import subprocess
import sys
import shutil
import tempfile
import stat

import publisher.settings as settings
from publisher.git_handling.history import get_commits_for_procedure, get_commits_for_phase
from publisher.git_handling.notes import clear_similar_historical_notes, edit_note
from publisher.utils import get_content_directories, WorkingDirectory, get_user_commit_selection, get_command_output, \
    get_procedure_code, get_yes_no, get_input, call_command_and_print_exception
from publisher.validators import validate_save
from publisher.exceptions import ProcedureCreationError


def build_channel_list():

    """ Lists all current branches of the repository
    """

    if os.path.exists(settings.CHANNELS_CHECKOUT_DIRECTORY):
        with WorkingDirectory(settings.CHANNELS_CHECKOUT_DIRECTORY):
            pull_and_merge()
        channels = []
        for item in os.listdir(settings.CHANNELS_CHECKOUT_DIRECTORY):
            name, extension = os.path.splitext(item)
            if extension in ['.yml']:
                channels.append(name)
        return channels
    else:
        raise Exception('You do not have the content-channels repository, please run the setup command.')


def build_procedure_list():

    """ Lists all current branches of the repository
    """

    def is_procedure(p):
        return p not in ['master', 'HEAD', '->'] and p.strip() != ''

    if os.path.exists(settings.PROCEDURE_CHECKOUT_DIRECTORY):
        with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
            print("Getting updates from repository")
            subprocess.check_output(['git', 'fetch', '--quiet'])
            print("Getting list of branches")
            output, _ = get_command_output(['git', 'branch', '-r'])
            procedure_list = output.split()
            new_procedure_list = []
            for name in procedure_list:
                if 'origin' in name:
                    new_procedure_list.append(name.split('/')[-1])
                else:
                    new_procedure_list.append(name)

            return [p for p in new_procedure_list if is_procedure(p)]

    else:
        raise Exception('You do not have the content-production repository, please run the setup command.')


def display_procedures():
    print('\n'.join(build_procedure_list()))


def build_phase_list():
    """ Returns a list of phases for the current module
    """
    procedure_directories = get_content_directories(settings.PROCEDURE_CHECKOUT_DIRECTORY)
    if len(procedure_directories) == 0:
        return []

    phase_folder = os.path.join(settings.PROCEDURE_CHECKOUT_DIRECTORY, procedure_directories[0])
    return get_content_directories(phase_folder)


def get_user_selected_phase():
    phases = sorted(build_phase_list())
    return get_user_commit_selection('\nSelect a phase to display its history (0 to cancel)', phases)


def display_phase_history(phase):
    print("\n".join([str(c) for c in get_commits_for_phase(phase)]))


def revert_phase_commit(phase, commit_id):
    """ Get the phase folders state at a certain commit and copy it into the latest version of the branch, unsaved.
    """
    current_procedure = get_procedure_code()
    temp_phase_dir = tempfile.mkdtemp()
    phase_path = os.path.join(settings.PROCEDURE_CHECKOUT_DIRECTORY, current_procedure, phase)

    with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
        print('Performing checkout of commit {}'.format(commit_id))
        call_command_and_print_exception(['git', 'checkout', '-q', commit_id], 'Could not checkout specified commit.')
        shutil.copytree(phase_path, os.path.join(temp_phase_dir, phase))
        print('Performing checkout of branch {}'.format(current_procedure))
        call_command_and_print_exception(['git', 'checkout', '-q', current_procedure],
                                         'Could not revert to latest commit.')
        shutil.rmtree(phase_path, onerror=remove_readonly)

        shutil.copytree(os.path.join(temp_phase_dir, phase), os.path.join(current_procedure, phase))
        shutil.rmtree(temp_phase_dir, onerror=remove_readonly)


def display_phase_history_and_revert(phase_code):
    if not phase_code:
        sys.exit(0)
    with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
        output, _ = get_command_output(['git', 'status'])
        if 'working tree clean' in output:
            message = "\nEnter the number of the version you want to revert to (0 to cancel):"
            user_input = get_user_commit_selection(message, get_commits_for_phase(phase_code))
            if not user_input:
                sys.exit(0)
            revert_phase_commit(phase_code, user_input.id)
            print(phase_code + " has been reverted to the selected commit.")
        else:
            print('You have unsaved changes, please commit them or perform a tspub reset before reverting to a '
                  'different commit.')


def reset_repo():
    """ Git reset
    """
    with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
        print('Performing git reset')
        call_command_and_print_exception(['git', 'reset', '--hard', '--quiet'], "Could not perform git reset.")
        print('Performing git clean')
        call_command_and_print_exception(['git', 'clean', '-f', '-d', '-q'], "Could not perform git clean.")


def cl_reset_repo():
    """ Git reset with user interaction
    """
    print("Resetting will lose any unsaved changes, are you sure you wish to continue? (y or n):")
    sys.stdout.flush()
    reset_confirmation = get_yes_no()
    if reset_confirmation:
        reset_repo()
    else:
        print("Operation cancelled.")


def remove_readonly(func, path, _):
    """Clear the readonly bit and reattempt the removal
    """
    os.chmod(path, stat.S_IWRITE)
    func(path)


def change_procedure(procedure):
    """Switches git branch to the procedure selected
    """

    for remote_procedure in build_procedure_list():
        if remote_procedure.lower() == procedure.lower():
            if remote_procedure == procedure:
                break
            else:
                print('\nYou typed in ' + procedure)
                print('We assume you meant ' + remote_procedure)
                print('Now changing to procedure: ' + remote_procedure + '\n')
                procedure = remote_procedure
                break

    with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
        command = ['git', '-c', 'filter.lfs.clean=', '-c', 'filter.lfs.smudge=', '-c', 'filter.lfs.process=', '-c',
                   'filter.lfs.required=false', 'checkout', '-q', procedure]
        message = 'Could not find the specified procedure. Make sure you have run setup and entered the correct ' \
                  'procedure name'
        print('Switching to procedure: {}'.format(procedure))
        call_command_and_print_exception(command, message)

        print("Retrieving procedure assets")
        subprocess.call(['git', '-c', 'filter.lfs.clean=', '-c', 'filter.lfs.smudge=', '-c', 'filter.lfs.process=',
                         '-c', 'filter.lfs.required=false', 'pull', '-q'])
        subprocess.call(['git', 'lfs', 'pull'])
        subprocess.call(['git', 'clean', '-xdfq'])

    with WorkingDirectory(settings.CHANNELS_CHECKOUT_DIRECTORY):
        print("Retrieving updates to channels repository")
        subprocess.call(['git', 'pull', '-q'])


def save_working_changes(message, initial=False, procedure_code=None):
    """Commits and pushes with the current changes to the repo
    """
    validate_save()

    try:
        with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
            if commit_working_changes(message, settings.PROCEDURE_CHECKOUT_DIRECTORY):
                if initial:
                    print('Performing initial push to repository')
                    subprocess.check_output(['git', 'push', '-q', '--set-upstream', 'origin', procedure_code])
                else:
                    pull_and_merge()
                    print('Pushing to repository')
                    subprocess.check_output(['git', 'push', '-q'])
            else:
                print('No changes detected.')

    except subprocess.CalledProcessError:
        print('Could not commit/push the changes.')
        raise


def save_channel_changes(message):
    """Commits and pushes with the current changes to the repo
    """

    try:
        with WorkingDirectory(settings.CHANNELS_CHECKOUT_DIRECTORY):
            if commit_working_changes(message, settings.CHANNELS_CHECKOUT_DIRECTORY):
                pull_and_merge()
                print('Pushing changes to repository')
                subprocess.check_output(['git', 'push', '-q'])
            else:
                print('No changes detected.')

    except subprocess.CalledProcessError:
        print('Could not commit/push the changes.')
        raise


def commit_working_changes(message, checkout_directory):

    with WorkingDirectory(checkout_directory):
        subprocess.check_output(['git', 'stage', '.'])
        output, _ = get_command_output(['git', 'commit', '-a', '-m', '"' + message + '"'])

    return 'nothing to commit' not in output


def publish(selected_commit):
    """ Publishes the procedure with a git note containing the distribution group to the chosen commit
    """

    all_commits = get_commits_for_procedure()
    with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):

        try:
            clear_similar_historical_notes(selected_commit, all_commits)
            edit_note(selected_commit)
        except Exception:
            print('Could not publish the procedure to the specified distribution group.')
            raise


def publish_with_display(dist_groups, number_of_commits_shown):

    with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
        dist_groups = dist_groups or "TS-Testing"  # Default distribution group
        number_of_commits_shown = number_of_commits_shown or 10  # Default number of commits shown

        print("Current module:")
        print(subprocess.check_output(['git', 'rev-parse', '--abbrev-ref', 'HEAD']))

        all_commits = get_commits_for_procedure()
        if not all_commits:
            raise Exception("Either there were no previous commits or you set the optional '--number' argument to be "
                            "zero. Please either save your work first or input a positive integer to the --number "
                            "argument.")

        display_commits = all_commits[:int(number_of_commits_shown)]
        message = "\nEnter the number of the version you want to publish here (0 to cancel):"
        selected_commit = get_user_commit_selection(message, display_commits)

        if selected_commit:
            dist_groups = dist_groups.replace(' ', '')
            publish(selected_commit.copy_with_new_note(dist_groups))


def delete_unstaged_changes():
    """Deletes all unstaged changes
    """
    try:
        with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
            print('Deleting local changes')
            subprocess.check_output(['git', 'checkout', '-q', '.'])
    except Exception:
        print("Could not delete local changes.")
        raise


def has_unstaged_changes():
    """Returns whether or not there are uncomitted changes
    """
    try:
        with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
            response, _ = get_command_output(['git', 'status', '-s'])
            return True if response else False
    except Exception:
        print("Could not retrieve the git diff. Please ensure the git repo is setup correctly")
        raise


def create_procedure_branch(procedure):
    """ Creates a new branch for the given procedure_code
    """
    for remote_procedure in build_procedure_list():
        if remote_procedure.lower() == procedure.lower():
            print('Git is case sensitive, the procedure {} already exists.\n The procedure you tried to create - {} - '
                  'has the same name.\n Please either correct the case of the procedure or create a new procedure with'
                  ' a different name.'.format(remote_procedure, procedure))
            raise ProcedureCreationError

    try:
        with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
            print('Switching to master branch')
            subprocess.check_output(['git', 'checkout', '-q', 'master'])
            print('Creating branch: {}'.format(procedure))
            subprocess.check_output(['git', 'branch', procedure])
            change_procedure(procedure)
    except Exception:
        print("Unable to create a new procedure with name {}".format(procedure))
        raise


def pull_and_merge():
    """ Try a git pull and request user input to decide how a merge conflict is resolved if there is one
    """
    # Find the author of any potential merge conflicts
    try:
        print('Performing git pull')
        subprocess.check_output(['git', 'pull', '-q'])
    except subprocess.CalledProcessError:
        # Ask the user which commit they would like to keep
        diff = subprocess.Popen(['git', 'diff'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        print(diff.communicate()[0])
        print("\n" + "There is a merge error.")
        print("Someone made a commit that conflicts with yours.")
        print("Would you like to accept your changes or theirs?")
        print("Type 'MINE' or 'THEIRS' to choose." + "\n")
        sys.stdout.flush()
        user_input = get_input()
        try:
            if user_input.lower() == "mine":
                subprocess.check_output(['git', 'checkout', '--ours', '.'])
                subprocess.check_output(['git', 'add', '.'])
                subprocess.check_output(['git', 'commit', '-a', '-m', 'merge'])
            elif user_input.lower() == "theirs":
                subprocess.check_output(['git', 'checkout', '--theirs', '.'])
                subprocess.check_output(['git', 'add', '.'])
                subprocess.check_output(['git', 'commit', '-a', '-m', 'merge'])
            else:
                print('\n' + 'Please enter either "MINE" or "THEIRS" to resolve the merge conflict.' + '\n')
                pull_and_merge()
        except subprocess.CalledProcessError:
            print('\n' + 'Something went wrong while trying to merge, please try again or contact pipeline for support'
                  + '\n')
            exit(1)



import os

import publisher.settings as settings
from publisher.utils import get_procedure_code


class ProcedureFileManager(object):

    def __init__(self, procedure):
        self._procedure_code = procedure.code
        self._ensure_structure_exists()

        self.phase_files = [PhaseFileManager(p, self._procedure_code) for p in procedure.phases]

    def _ensure_structure_exists(self):
        for d in [self.base_directory, self.asset_directory, self.build_asset_directory, self.more_info_directory,
                  self.overview_directory, self.devices_directory, self.eula_directory, self.csv_directory]:
            if not os.path.exists(d):
                os.makedirs(d)

    @property
    def base_directory(self):
        return os.path.join(settings.PROCEDURE_CHECKOUT_DIRECTORY, self._procedure_code)

    @property
    def procedure_file(self):
        return os.path.join(self.base_directory, 'procedure.yml')

    @property
    def asset_directory(self):
        return os.path.join(self.base_directory, 'assets')

    @property
    def build_asset_directory(self):
        return os.path.join(self.base_directory, 'build_assets')

    @property
    def more_info_directory(self):
        return os.path.join(self.build_asset_directory, 'moreInfo')

    @property
    def overview_directory(self):
        return os.path.join(self.build_asset_directory, 'overview')

    @property
    def devices_directory(self):
        return os.path.join(self.build_asset_directory, 'devices')

    @property
    def eula_directory(self):
        return os.path.join(self.build_asset_directory, 'eula')

    @property
    def csv_directory(self):
        return os.path.join(self.build_asset_directory, 'csv')


class PhaseFileManager(object):

    def __init__(self, phase, procedure_code):
        self._procedure_code = procedure_code
        self.phase_code = phase.code

        self._ensure_structure_exists()

    def _ensure_structure_exists(self):
        for d in [self.base_directory, self.asset_directory, self.translation_directory]:
            if not os.path.exists(d):
                os.makedirs(d)

    @property
    def base_directory(self):
        return os.path.join(self.procedure_directory, self.phase_code)

    @property
    def procedure_directory(self):
        return os.path.join(settings.PROCEDURE_CHECKOUT_DIRECTORY, self._procedure_code)

    @property
    def phase_file(self):
        return os.path.join(self.base_directory, 'phase.yml')

    @property
    def asset_directory(self):
        return os.path.join(self.base_directory, 'assets')

    @property
    def translation_directory(self):
        return os.path.join(self.base_directory, 'translations')


class DeliveryFileManager(object):

    def __init__(self, procedure_code, phases, vbs):
        self._procedure_code = procedure_code
        self.phases = phases
        self.vbs = vbs
        self.phase_directories = {}

        if self.vbs:
            self.base_directory = settings.VBS_ROOT
        else:
            self.base_directory = settings.CG_APP5_ROOT

        self._ensure_procedure_structure_exists()
        self._ensure_phase_structure_exists()

    def _ensure_procedure_structure_exists(self):
        for d in [self.procedure_directory, self.proc_thumbnail_directory]:
            if not os.path.exists(d):
                os.makedirs(d)

    def _ensure_phase_structure_exists(self):
        for phase in self.phases:
            self.phase_code = phase.code
            for d in [self.phase_directory, self.phase_thumbnail_directory, self.graphics_directory, self.pip_directory,
                      self.project_directory, self.widget_directory]:
                if not os.path.exists(d):
                    os.makedirs(d)
                self.phase_directories[self.phase_code] = {
                    "base_directory": self.phase_directory,
                    "thumbnail_directory": self.phase_thumbnail_directory,
                    "graphics_directory": self.graphics_directory,
                    "pip_directory": self.pip_directory,
                    "project_directory": self.project_directory,
                    "widget_directory": self.widget_directory,
                }

    @property
    def phase_directory(self):
        return os.path.join(self.base_directory, self.phase_code)

    @property
    def procedure_directory(self):
        return os.path.join(self.base_directory, self._procedure_code)

    @property
    def proc_thumbnail_directory(self):
        return os.path.join(self.procedure_directory, 'thumbnails')

    @property
    def phase_thumbnail_directory(self):
        return os.path.join(self.phase_directory, 'thumbnails')

    @property
    def graphics_directory(self):
        return os.path.join(self.phase_directory, 'graphics')

    @property
    def pip_directory(self):
        return os.path.join(self.phase_directory, 'pipSource')

    @property
    def project_directory(self):
        return os.path.join(self.phase_directory, 'projectFiles')

    @property
    def widget_directory(self):
        return os.path.join(self.phase_directory, 'widgetGraphics')


def get_procedure_yaml_file():
    return os.path.join(settings.PROCEDURE_CHECKOUT_DIRECTORY, get_procedure_code(), 'procedure.yml')


def get_all_procedure_yaml_files():

    ignore_directories = ['build_assets', 'assets', 'translations']

    yaml_files = []
    for subdir, dirs, files in os.walk(settings.PROCEDURE_CHECKOUT_DIRECTORY):

        directories = os.path.normpath(subdir).split(os.sep)
        if any(d in directories for d in ignore_directories) or len([d for d in directories if d.startswith('.')]) > 0:
            continue

        yaml_files += [os.path.join(subdir, f) for f in files if f.endswith('.yml')]

    return yaml_files



from __future__ import absolute_import, division, print_function, unicode_literals

import os
import platform

GIT_DIRECTORY = os.path.expanduser("~/git")
PROCEDURE_CHECKOUT_DIRECTORY = os.path.expanduser("~/git/content-procedures")
CHANNELS_CHECKOUT_DIRECTORY = os.path.expanduser("~/git/content-channels")
TRANSLATIONS_CHECKOUT_DIRECTORY = os.path.expanduser("~/git/content-translations")

STUDIO_GIT_PATH = 'studio-git.touchsurgery.com'
PROCEDURE_REPOSITORY = '{0}:/srv/git/procedure-repo'.format(STUDIO_GIT_PATH)
CHANNELS_REPOSITORY = '{0}:/srv/git/channel-repo'.format(STUDIO_GIT_PATH)
TRANSLATIONS_REPOSITORY = '{0}:/srv/git/translation-repo'.format(STUDIO_GIT_PATH)

TEMPLATE_DIRECTORY = os.path.join(os.path.dirname(__file__), "template_files")

SSH_DIRECTORY_PATH = os.path.expanduser('~/.ssh')
SSH_CONFIG_PATH = os.path.expanduser("~/.ssh/config")
RSA_PUBLIC_KEY_PATH = os.path.expanduser('~/.ssh/touchsurgery_publisher_rsa.pub')
RSA_PRIVATE_KEY_PATH = os.path.expanduser('~/.ssh/touchsurgery_publisher_rsa')

if platform.system() == "Windows":
    BASE_DATA_DIR = "Z:\\"
else:
    BASE_DATA_DIR = os.path.join("/Volumes", "content")

PRODUCTION_INFO_DIR = os.path.join(BASE_DATA_DIR, "productionInfo")
PRODUCTION_INFO_BACKUP_DIR = os.path.expanduser("~/git/production-info")
CHANNELS_INFO_DIR = CHANNELS_CHECKOUT_DIRECTORY

CONTENT_DB_DIR = os.path.join(BASE_DATA_DIR, "assetdb", ".db", "contentdb.sqlite3")

ORIGINAL_CONTENT_ROOT = "C:/TouchSurgery/assetdb/vault2"
ORIGINAL_CONTENT_ROOT_OLD = "C:/TouchSurgery/assetdb"
if platform.system() == "Windows":
    REPLACEMENT_CONTENT_ROOT = "C:/TouchSurgery/assetdb/vault2"
    REPLACEMENT_CONTENT_ROOT_OLD = "C:/TouchSurgery/assetdb"
else:
    REPLACEMENT_CONTENT_ROOT = "/Volumes/content/assetdb/vault2"
    REPLACEMENT_CONTENT_ROOT_OLD = "/Volumes/content/assetdb"

CG_APP5_ROOT = os.path.join(BASE_DATA_DIR, "delivery", "app5")
VBS_ROOT = os.path.join(BASE_DATA_DIR, "delivery", "vbs")

LOCALE_DIRECTORIES = {
    'en': 'en',
    'es': 'es',
    'pt': 'pt',
    'ru': 'ru',
    'zh-hans': 'hans',
    'zh-hant': 'hant',
    'ja': 'ja'
}

CHANNEL_TYPES = ['touchsurgery', 'institution', 'company']
DEFAULT_SCALE = "scale='768:1024'"  # fix frame-popping in app5. 680:1024 previously



from __future__ import absolute_import, division, print_function, unicode_literals

import os.path

from publisher.exceptions import ContentMissingError
from publisher.processing.data_sources.utils import YamlObject
import publisher.settings as settings


def get_more_info_data(info_id, code):
    yaml_dir = os.path.join(settings.PROCEDURE_CHECKOUT_DIRECTORY, code, "build_assets", "moreInfo", info_id)

    yaml_file = os.path.join(yaml_dir, info_id + ".yml")
    source_asset_dir = os.path.join(yaml_dir, "assets")

    if not os.path.isfile(yaml_file):
        raise ContentMissingError("Yaml infocard not found in {0}".format(yaml_file))

    print("-- Found more info card: %s --" % info_id)
    return read_yaml_data(yaml_file), source_asset_dir


def read_yaml_data(yaml_file):
    yaml = YamlObject()

    with open(yaml_file, "r") as overview_file:
        overview_data = yaml.load(overview_file)
        return overview_data



from __future__ import absolute_import, division, print_function, unicode_literals


def get_author_from_overview(overview):
    authors_page = [page for page in overview if "authors" in page["information_card"]["title"].lower()]
    if not authors_page:
        return ""

    authors_section = [section for section in authors_page[0]["information_card"]["sections"]
                       if "author" in section["title"].lower()]
    if not authors_section:
        return ""

    authors = [author["author"]["name"].split(",")[0] for author in authors_section[0]["body"]]
    if not authors:
        return ""

    return ", ".join(authors)
